@@ -30,7 +30,7 @@ use crate::{
 pub fn create<I, S>(
     info_sender: Sender<I>,
     waveforms_file_location: &Path,
-    pitch_wheel_sensivity: Ratio,
+    pitch_wheel_sensitivity: Ratio,
     cc_numbers: ControlChangeNumbers,
     num_buffers: usize,
     buffer_size: usize,
@@ -42,7 +42,7 @@ pub fn create<I, S>(
         },
         magnetron: Magnetron::new(num_buffers, 2 * buffer_size), // The first invocation of cpal uses the double buffer size
         damper_pedal_pressure: 0.0,
-        pitch_wheel_sensivity,
+        pitch_wheel_sensitivity,
         pitch_bend: Ratio::default(),
         last_id: 0,
     };
@@ -247,7 +247,7 @@ struct SynthState<S> {
     storage: ControlStorage,
     magnetron: Magnetron,
     damper_pedal_pressure: f64,
-    pitch_wheel_sensivity: Ratio,
+    pitch_wheel_sensitivity: Ratio,
     pitch_bend: Ratio,
     last_id: u64,
 }
@@ -259,38 +259,42 @@ enum WaveformState<S> {
 }
 
 impl<S: Eq + Hash> WaveformSynth<S> {
-    pub fn write(&mut self, buffer: &mut [f64], audio_in: &mut Consumer<f32>) {
+    pub fn write(&mut self, buffer: &mut [f64], audio_in: &mut Consumer<f64>) {
         for message in self.messages.try_iter() {
             self.state.process_message(message)
         }
 
         let sample_width_secs = 1.0 / audio::DEFAULT_SAMPLE_RATE;
 
-        let SynthState {
-            playing,
-            storage,
-            magnetron,
-            damper_pedal_pressure,
-            pitch_bend,
-            ..
-        } = &mut self.state;
-        let envelope_map = &self.envelope_map;
+        self.state.magnetron.clear(buffer.len() / 2);
+        self.state
+            .magnetron
+            .set_audio_in(buffer.len() / 2, audio_in);
 
-        magnetron.clear(buffer.len() / 2);
-        magnetron.set_audio_in(audio_in);
-
-        playing.retain(|id, waveform| {
+        self.state.playing.retain(|id, waveform| {
             let key_hold = match id {
                 WaveformState::Stable(_) => {
-                    waveform.properties.pitch_bend = *pitch_bend;
+                    waveform.properties.pitch_bend = self.state.pitch_bend;
                     1.0
                 }
-                WaveformState::Fading(_) => *damper_pedal_pressure,
+                WaveformState::Fading(_) => self.state.damper_pedal_pressure,
             };
-            magnetron.write(waveform, envelope_map, storage, key_hold, sample_width_secs)
+            self.state.magnetron.write(
+                waveform,
+                &self.envelope_map,
+                &self.state.storage,
+                key_hold,
+                sample_width_secs,
+            )
         });
 
-        for (&out, target) in magnetron.total().iter().zip(buffer.chunks_mut(2)) {
+        for (&out, target) in self
+            .state
+            .magnetron
+            .total()
+            .iter()
+            .zip(buffer.chunks_mut(2))
+        {
             if let [left, right] = target {
                 *left += out / 10.0;
                 *right += out / 10.0;
@@ -329,7 +333,7 @@ impl<S: Eq + Hash> SynthState<S> {
                 self.damper_pedal_pressure = curve;
             }
             Message::PitchBend { bend_level } => {
-                self.pitch_bend = self.pitch_wheel_sensivity.repeated(bend_level);
+                self.pitch_bend = self.pitch_wheel_sensitivity.repeated(bend_level);
             }
             Message::Control { control, value } => {
                 self.storage.write(control, value);
